unit als_318;

{Public domain unit to drive the als318 9-digit 7-segment LED panel
with k555id10 binary-to-decimal convertor as horizontal
scan from LPT port.
The unit lets you make programs that show different characters
(binary, decimal and hexademical numbers, some Latin and Cyrillic
letters, other stuff), and even simple animation using als318.
Your programs can implement clocks, calculators, games, demos, etc.
The device is simple to build and uses very cheap components.
See the diagram in the file als_318.png
Please compile the unit using TMT Pascal (Turbo Pascal cannot compile it
because it finds some errors here :-)
Have fun!
NO WARRANTY - USE AT YOUR OWN RISK}

interface
uses crt;

procedure initport; {initializes the LPT port}
procedure sendals(segments,place:byte); {sends the digit into als318}
function tranchar(symb:char):byte;
{transcodes a character into 7-segment code}
procedure sendline(segmline:array[0..8] of byte);
{sends a line of 9 bytes into
als318}

implementation

const
 lptport:integer=888; {main LPT port decimal address}
 dopport:integer=890; {additional control port decimal address}
 transcd:array[0..8] of byte=(11,10,9,8,15,14,13,12,3);
 {digit place transcoding values}

 {Now it's time to make some transcoding tables}

 numbers:array[0..9] of char=('0','1','2','3','4','5','6','7','8','9');
 {decimal and binary number figures}
 tnumbers:array[0..9] of byte=(63,6,91,79,102,109,125,39,127,111);
 {the same, transcoded into the 7-segment code}
 hexnums:array[10..15] of char=('a','b','c','d','e','f');
 {additional hexademical number figures as Latin letters from a to f}
 thexnums:array[10..15] of byte=(95,124,88,94,123,113);
 {the same, transcoded into the 7-segment code}
 latincs:array[1..20] of char=('A','B','C','D','E','F','G','H','I','J','L','N','O','P','Q','R','S','T','U','Y');
 {some Latin capital characters which are showable on 7-segment basis}
 tlatincs:array[1..20] of byte=(119,127,57,94,121,113,61,118,48,14,56,55,63,115,103,49,109,7,62,110);
 {the same, transcoded into the 7-segment code}
 latinls:array[7..20] of char=('g','h','i','j','l','n','o','p','q','r','s','t','u','y');
 {some Latin lowercase characters which are showable on 7-segment basis}
 tlatinls:array[7..20] of byte=(111,116,16,12,56,84,92,115,103,80,109,120,28,110);
 {the same, transcoded into the 7-segment code}
 cyrilcs:array[1..21] of char=('','','','','','','','','','','','','','','','','','','','','');
 {some cp866 Cyrillic capital characters which are showable on 7-segment basis}
 tcyrilcs:array[1..21] of byte=(119,125,127,49,95,121,79,62,29,86,118,63,55,115,57,7,110,102,32,124,79);
 {the same, transcoded into the 7-segment code}
 cyrills:array[1..21] of char=('','','','','','','','','','','','','','','','','','','','','');
 {some cp866 Cyrillic lowercase letters which are showable on 7-segment basis}
 tcyrills:array[1..21] of byte=(95,125,127,80,95,123,79,28,93,86,118,92,84,115,88,68,110,102,32,124,79);
 misc:array[1..11] of char=(' ',',','.','[',']','_','-','=','"','!','?');
 {miscellanous characters like space and comma}
 tmisc:array[1..11] of byte=(0,4,128,57,15,8,64,65,34,134,221);
 {the same, transcoded into the 7-segment code}

var
 i:integer; {cycle variable}

procedure initport;
begin
 port[lptport]:=0;
 port[dopport]:=0
end;

procedure sendals(segments,place:byte);
var where:byte;
begin
 where:=place;
 if where<=8 then where:=transcd[where];
 port[lptport]:=segments;
 port[dopport]:=where;
 delay(1)
end;

function tranchar(symb:char):byte;
var segments:byte;
begin
 segments:=ord(symb);
 {if the character is not found in all tables, its code is sent instead}
 for i:=0 to 9 do if symb=numbers[i] then segments:=tnumbers[i];
 for i:=10 to 15 do if symb=hexnums[i] then segments:=thexnums[i];
 for i:=1 to 20 do if symb=latincs[i] then segments:=tlatincs[i];
 for i:=7 to 20 do if symb=latinls[i] then segments:=tlatinls[i];
 for i:=1 to 21 do if symb=cyrilcs[i] then segments:=tcyrilcs[i];
 for i:=1 to 21 do if symb=cyrills[i] then segments:=tcyrills[i];
 for i:=1 to 11 do if symb=misc[i] then segments:=tmisc[i];
 tranchar:=segments
end;

procedure sendline(segmline:array[0..8] of byte);
begin
 for i:=0 to 8 do sendals(segmline[i],i)
end;

end.